/*
    AQTRONIX C++ Library
    Copyright 2015 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
#pragma once

#include "Settings.h"
#include "StringCache.h"
#include "RegexCache.h"

class RuleParameters{
public:
	Rule Pollution;
	RuleString NameRequireRegex;
	Rule InputValidation;
};

class WebParam{
public:
	CStringCache Cache;
	bool AllowNotInList;
	bool IgnoreCase;
	CMapStringToString Validators;
	CMapStringToString MaxLength;

	void Init(int hours = 72, unsigned int maxitems = 10)
	{
		Cache.SetMaxItems(maxitems);
		Cache.SetMaxAge(CTimeSpan(0,hours,0,0));
	}

	void Clear()
	{
		Cache.Clear();
	}

	void ApplyConstraints()
	{
		if(IgnoreCase){
			ApplyIgnoreCase(Validators);
			ApplyIgnoreCase(MaxLength);
		}
	}

private:
	void ApplyIgnoreCase(CMapStringToString& map)
	{
		CMapStringToString convert;
		CString key;
		CString val;
		POSITION pos = map.GetStartPosition();
		while(pos!=NULL){
			map.GetNextAssoc(pos,key,val);
			CStringHelper::Safe_MakeLower(key);
			convert.SetAt(key,val);
		}
		CStringHelper::Copy(convert,map,true);
	}
};

class CParameters : public CSettings
{
private:
	void AddValidator(LPCTSTR Name, LPCTSTR Pattern);

public:
	WebParam Query;
	WebParam Post;
	WebParam Cookie;
	WebParam Headers;

	class _Validation{
	public:
		CMapStringToString Patterns;
		CStringList Order;
	}Validation;

	CRegexCache Regex;

	CString MatchPattern(CString& data);
	bool Validate(WebParam& Parameters, CString& name, CString& value);
	bool Validate(WebParam& Parameters, CString& name, int length);
	bool HasValidator(WebParam& Parameters, CString name);

	CString ToXML();
	bool ParseXML(CString& key, CString& val);
	bool WriteToFileINI(LPCSTR fn);
	bool ReadFromFileINI(LPCTSTR fn);
	bool WriteToFileXML(LPCTSTR fn);
	void LoadDefaults();
	bool ApplyConstraints();

	void Init();
	void Clear();

	CParameters(void);
	virtual ~CParameters(void);
};
